// DynWindow Object
// a widget-object that creates a draggable window
// 2000.03.11

// Copyright (C) 2000 Dan Steinman
// Redistributable under the terms of the GNU Library General Public License
// Available at http://www.dansteinman.com/dynapi/

function DynWindow(title) {
	this.title = title
	this.component = new DynLayer()
	this.component.dw = this
	this.component.onCreate = function() {
		this.dw.setTitle(this.dw.title)
	}

	this.titleBar = this.component.addChild(new DynLayer())
	this.buttonIcon = this.titleBar.addChild(new DynLayer())
	this.titleBarGrip = this.titleBar.addChild(new DynLayer())
	this.titleText = this.titleBar.addChild(new DynLayer())
	
	this.borderT = this.component.addChild(new DynLayer())
	this.borderB = this.component.addChild(new DynLayer())
	this.borderL = this.component.addChild(new DynLayer())
	this.borderR = this.component.addChild(new DynLayer())
		
	this.cornerTL = this.component.addChild(new DynLayer())
	this.cornerTR = this.component.addChild(new DynLayer())
	this.cornerBL = this.component.addChild(new DynLayer())
	this.cornerBR = this.component.addChild(new DynLayer())
	
	this.cornerResize = this.component.addChild(new DynLayer('',0,0,25,25))
	
	this.buttonClose = new ButtonImage()
	this.component.addChild(this.buttonClose)
		
	this.buttonMax = new ButtonImage()
	this.buttonMax.checkbox = true
	this.component.addChild(this.buttonMax)
	
	this.buttonMin = new ButtonImage()
	this.component.addChild(this.buttonMin)
	
	this.content = this.component.addChild(new DynLayer())
	this.content.setBgColor('#ffffff')
	
	// events

	this.buttonCloseEvent = new EventListener(this)
		this.buttonCloseEvent.onmousedown = function(e) {
			e.getTarget().invokeEvent("close")
		}
		this.buttonClose.addEventListener(this.buttonCloseEvent)

	this.buttonMaxEvents = new EventListener(this)
		this.buttonMaxEvents.onselect = function(e) {
			if (!e.getTarget().maximized) e.getTarget().setMaximized(true)
		}
		this.buttonMaxEvents.ondeselect = function(e) {
			if (e.getTarget().maximized) e.getTarget().setMaximized(false)
		}
		this.buttonMax.addEventListener(this.buttonMaxEvents)

	this.buttonMinEvent = new EventListener(this)
		this.buttonMinEvent.onmousedown = function(e) {
			e.getTarget().setMinimized(true)
		}
		this.buttonMin.addEventListener(this.buttonMinEvent)

	this.contentEvent = new EventListener(this)
		this.contentEvent.onmousedown = function(e) {
			e.getTarget().setFocused(true)
			e.getTarget().invokeEvent("focus")
			e.setBubble(false)
		}
		this.content.addEventListener(this.contentEvent)

	this.focusEvent = new EventListener(this)
		this.focusEvent.onmousedown = function(e) {
			e.getTarget().setFocused(true)
			e.getTarget().invokeEvent("focus")
		}
		this.component.addEventListener(this.focusEvent)
	
	this.shadeEvent = new EventListener(this)
		this.shadeEvent.ondblclick = function(e) {
			var o = e.getTarget()
			o.setShaded(!o.shaded)
			e.setBubble(true)
		}
		this.titleBar.addEventListener(this.shadeEvent)
	
	this.dragResizeEvent = new EventListener(this)
		this.dragResizeEvent.ondragmove = function(e) {
			var o = e.getTarget()
			var w = o.cornerResize.getPageX()+25-o.component.getPageX()
			var h = o.cornerResize.getPageY()+25-o.component.getPageY()
			if (w<o.minW) w = o.minW
			if (h<o.minH) h = o.minH
			o.maximized = false
			o.shaded = false
			o.setOutterSize(w,h)
		}
		this.dragResizeEvent.ondragend = function(e) {
			e.getTarget().invokeEvent("resize")
		}
		this.cornerResize.addEventListener(this.dragResizeEvent)
    	this.dragEventListener = new EventListener(this.content)
    	this.dragEventListener.ondragstart=function(e) {
    	    var o=e.getTarget();
    	    o.setVisible(false);
    	}
    	this.dragEventListener.ondragend=function(e) {
    	    var o=e.getTarget();
    	    o.setVisible(true);
    	}
		this.component.addEventListener(this.dragEventListener)
	this.setResizable(this.resizable)
}

WidgetInterface.inherit(DynWindow)

DynWindow.prototype.w = 0
DynWindow.prototype.h = 0
DynWindow.prototype.themeSet = false
DynWindow.prototype.resizable = true

DynWindow.prototype.setTheme = function(theme) {
	// dimensions are based on the image sizes
	
	this.theme = theme
	this.images = theme.images
	this.style = theme.style
	this.minW = theme.minW
	this.minH = theme.minH
	
	var l = this.images['l'].w
	var r = this.images['r'].w
	var t = this.images['t'].h
	var b = this.images['b'].h
	var titleH = this.images['titlebg0'].h
	var cx = l
	var cy = t+titleH
	var cw = this.w-l-r
	var ch = this.h-t-b-titleH
	var corner = this.images['cornerTL'].w

	this.titleBar.moveTo(l,t)
	this.titleBar.setBgImage(this.images['titlebg0'].src)
	this.buttonIcon.setSize(this.images['icon'].w,this.images['icon'].h)
	this.buttonIcon.setBgImage(this.images['icon'].src)
	this.titleText.moveTo(this.images['icon'].w,0)
	this.titleBarGrip.moveTo(this.images['icon'].w,0)
	this.titleBarGrip.setBgImage(this.images['grip'].src)
	
	this.borderT.moveTo(cx,0)
	this.borderL.moveTo(0,t)
		
	this.borderT.setBgImage(this.images['t'].src)
	this.borderB.setBgImage(this.images['b'].src)
	this.borderL.setBgImage(this.images['l'].src)
	this.borderR.setBgImage(this.images['r'].src)
	
	this.cornerTL.setSize(this.images['cornerTL'].w,this.images['cornerTL'].h)
	this.cornerTR.setSize(this.images['cornerTR'].w,this.images['cornerTR'].h)
	this.cornerBL.setSize(this.images['cornerBL'].w,this.images['cornerBL'].h)
	this.cornerBR.setSize(this.images['cornerBR'].w,this.images['cornerBR'].h)
	
	this.cornerTL.setBgImage(this.images['cornerTL'].src)
	this.cornerTR.setBgImage(this.images['cornerTR'].src)
	this.cornerBL.setBgImage(this.images['cornerBL'].src)
	this.cornerBR.setBgImage(this.images['cornerBR'].src)
	
	this.buttonClose.setImages(this.images['close0'],this.images['close1'])
	this.buttonMax.setImages(this.images['max0'],this.images['max1'],this.images['restore0'],this.images['restore1'])
	this.buttonMin.setImages(this.images['min0'],this.images['min1'])
		
	this.content.moveTo(cx,cy)
	this.content.setSize(cw,ch)

	this.themeSet = true
}
DynWindow.prototype.setOutterSize = function(w,h) {
	this.w = w
//	this.h = h
	
	if (!this.themeSet) return
	
	if (!h) h=this.minH;

	this.h = h
	var l = this.images['l'].w
	var r = this.images['r'].w
	var t = this.images['t'].h
	var b = this.images['b'].h
	var titleH = this.images['titlebg0'].h
	var cx = l
	var cy = t+titleH
	var cw = this.w-l-r
	var ch = this.h-t-b-titleH
	
	this.component.setSize(w,h)
	this.content.setSize(cw,ch)
	
	this.titleBar.setSize(cw,titleH)
	this.titleText.setSize(Math.min(this.titleText.getContentWidth(),cw-this.images['icon'].w-this.images['close0'].w-this.images['max0'].w-this.images['min0'].w),titleH)
	this.titleBarGrip.setSize(cw-this.images['icon'].w-this.images['close0'].w-this.images['max0'].w-this.images['min0'].w-this.titleText.w,titleH)
	this.titleBarGrip.moveTo(this.titleText.x+this.titleText.w)
	
	this.borderT.setSize(cw,t)
	this.borderB.setSize(cw,b)
	this.borderL.setSize(l,this.h-t-b)
	this.borderR.setSize(r,this.h-t-b)
	
	this.borderR.moveTo(this.w-r,t)
	this.borderB.moveTo(cx,this.h-b)

	this.buttonClose.moveTo(this.w-r-this.images['close0'].w,t)
	this.buttonMax.moveTo(this.w-r-this.images['close0'].w-this.images['max0'].w,t)
	this.buttonMin.moveTo(this.w-r-this.images['close0'].w-this.images['max0'].w-this.images['min0'].w,t)
	
	this.cornerResize.moveTo(this.w-25,this.h-25)

	this.cornerTR.moveTo(this.w-this.images['cornerTR'].w,0)
	this.cornerBL.moveTo(0,this.h-this.images['cornerBL'].h)
	this.cornerBR.moveTo(this.w-this.images['cornerBR'].w,this.h-this.images['cornerBR'].h)
}
DynWindow.prototype.setInnerSize = function(w,h) {
	this.setOutterSize(w+this.images['l'].w+this.images['r'].w,h+this.images['t'].h+this.images['b'].h)
	this.invokeEvent("resize")
}
DynWindow.prototype.setSize = function(w,h) {
	this.setOutterSize(w,h)
	this.invokeEvent("resize")
}
DynWindow.prototype.setMinimized = function(b) {
    this.minimized=b;
	if (this.minimized) {
		this.component.setVisible(false)
		this.invokeEvent("minimize")
	} else {
		this.component.setVisible(true)
		this.invokeEvent("restore")
	}
}
DynWindow.prototype.setMaximized = function(b) {
    this.maximized=b;
	if (this.maximized) {
		this.origX = this.component.x
		this.origY = this.component.y
		this.origW = this.component.w
		this.origH = this.component.h
		this.component.moveTo(0,0)
		if (!this.buttonMax.selected) this.buttonMax.setSelected(true)
		this.setOutterSize(DynAPI.document.getWidth(),DynAPI.document.getHeight())
		this.invokeEvent("maximize")
		this.invokeEvent("resize")
	} else if (!this.minimized) {
		this.component.moveTo(this.origX,this.origY)
		this.setOutterSize(this.origW,this.origH)
		this.maximized = false
		this.invokeEvent("resize")
	}
}
DynWindow.prototype.setShaded = function(b) {
    this.shaded=b;
	if (this.shaded) {
		this.origH = this.component.h
		this.setOutterSize(this.component.w,this.minH)
	} else this.setOutterSize(this.component.w,this.origH)
}
DynWindow.prototype.getInnerWidth = function() {return this.content.w}
DynWindow.prototype.getInnerHeight = function() {return this.content.h}
DynWindow.prototype.setTitle = function (title) {
	this.title = title||''
	if (!this.themeSet) return
	this.titleText.setHTML('<nobr><font color="'+this.style.titleColor+'" family="'+this.style.titleFont+'" point-size="'+this.style.titleSize+'">'+this.title+'&nbsp;</span></nobr>')
	this.titleText.setSize(Math.min(this.titleText.getContentWidth(),this.w-this.images['l'].w-this.images['r'].w-this.images['min0'].w-this.images['min0'].w-this.images['max0'].w-this.images['close0'].w),this.images['titlebg0'].h)
	this.titleText.moveTo(null,(this.images['titlebg0'].h-this.titleText.getContentHeight())/2)
	this.titleBarGrip.setWidth(this.titleBar.w-this.images['icon'].w-this.titleText.w-this.images['min0'].w-this.images['max0'].w-this.images['close0'].w)
	this.titleBarGrip.moveTo(this.titleText.x+this.titleText.w)
}
DynWindow.prototype.getTitle = function(title) {return this.title}
DynWindow.prototype.setResizable = function (b) {
    this.resizable=b;
	if (this.resizable) DragEvent.enableDragEvents(this.cornerResize)
	else DragEvent.disableDragEvents(this.cornerResize)
}
DynWindow.prototype.getResizable = function() {return this.resizable}
DynWindow.prototype.setAlwaysOnTop = function (b) {
	this.alwaysOnTop = b
}
DynWindow.prototype.setFocused = function (b) {
    this.focused=b;
	if (this.focused) this.titleBar.setBgImage(this.images['titlebg0'].src)
	else this.titleBar.setBgImage(this.images['titlebg1'].src)
}
DynWindow.prototype.addChild = function(widget) {
	this.content.addChild(widget)
}
DynWindow.prototype.removeChild = function(widget) {
	this.content.removeChild(widget)
}
DynWindow.prototype.destroyWindow = function() {
	this.component.deleteFromParent()
}

function DynWindowManager(parentdoc) {
	if (!parentdoc) this.parent = DynAPI.document
	this.focusListener = new EventListener(this)
	this.focusListener.onfocus = function(e) {
	    if (e.getTarget() && e.getTarget().deFocus)
		    e.getTarget().deFocus(e.getSource())
	}
}
DynWindowManager.prototype.windows = []
DynWindowManager.prototype.defaultW = 200
DynWindowManager.prototype.defaultH = 200

DynWindowManager.prototype.loadTheme = function(theme,dir) {	
	this[theme] = new Object()
	var i = this[theme].images = new Object()
	i['l'] = DynAPI.getImage(dir+'border-v.gif',6,0)
	i['r'] = DynAPI.getImage(dir+'border-v.gif',6,0)
	i['t'] = DynAPI.getImage(dir+'border-h.gif',0,6)
	i['b'] = DynAPI.getImage(dir+'border-h.gif',0,6)
	i['icon'] = DynAPI.getImage(dir+'icon.gif',20,20)
	i['titlebg0'] = DynAPI.getImage(dir+'title-bg0.gif',0,21)
	i['titlebg1'] = DynAPI.getImage(dir+'title-bg1.gif',0,21)
	i['grip'] = DynAPI.getImage(dir+'title-grip.gif',0,21)
	i['max0'] = DynAPI.getImage(dir+'max0.gif',20,20)
	i['max1'] = DynAPI.getImage(dir+'max1.gif',20,20)
	i['restore0'] = DynAPI.getImage(dir+'restore0.gif',20,20)
	i['restore1'] = DynAPI.getImage(dir+'restore1.gif',20,20)
	i['min0'] = DynAPI.getImage(dir+'min0.gif',20,20)
	i['min1'] = DynAPI.getImage(dir+'min1.gif',20,20)
	i['close0'] = DynAPI.getImage(dir+'close0.gif',20,20)
	i['close1'] = DynAPI.getImage(dir+'close1.gif',20,20)
	i['cornerTL'] = DynAPI.getImage(dir+'corner-tl.gif',16,16)
	i['cornerTR'] = DynAPI.getImage(dir+'corner-tr.gif',16,16)
	i['cornerBL'] = DynAPI.getImage(dir+'corner-bl.gif',16,16)
	i['cornerBR'] = DynAPI.getImage(dir+'corner-br.gif',16,16)

	var s = this[theme].style = new Object()
	s.titleFont = ''
	s.titleSize = ''
	s.titleColor = '#000000'
	this[theme].minW = 90
	this[theme].minH = 32
	
	this["defaultTheme"] = this[theme]
}
DynWindowManager.prototype.addWindow = function(win,theme) {
	if (!theme) theme = this["defaultTheme"]
	this.windows[this.windows.length] = win

	win.setTheme(theme)
	win.setSize(this.defaultW,this.defaultH)
	win.moveTo(DynAPI.document.getWidth()/2-win.w/2,DynAPI.document.getHeight()/2-win.h/2)
	win.setVisible(true)
	win.wm = this
		
	win.addEventListener(this.focusListener)
	DragEvent.enableDragEvents(win)
	DragEvent.setDragBoundary(win)
	
	this.parent.addChild(win)
	return win
}
DynWindowManager.prototype.deleteWindow = function(win) {
	win.removeEventListener(this.focusListener)
	DragEvent.disableDragEvents(win)
	
	win.deleteFromParent()
	
	this.windows = DynAPI.removeFromArray(this.windows,win)
}
DynWindowManager.prototype.deleteAllWindows = function() {
	while(this.windows.length>0) {
		this.destroyWindow(this.windows[this.windows.length-1])
	}
}
DynWindowManager.prototype.focusWindow = function(win) {
	this.deFocusWindows(win)
}
DynWindowManager.prototype.deFocusWindows = function(omit) {
	for (var i in this.windows) {
		var win = this.windows[i]
		if (win!=omit && !win.alwaysOnTop && win.focused) {
			win.setFocused(false)
		}
	}
}
DynWindowManager.prototype.minimizeAll = function() {
	for (var i in this.windows) {
		this.windows[i].setMinimized(true)
	}
}
DynWindowManager.prototype.deminimizeAll = function() {
	for (var i in this.windows) {
		this.windows[i].setMinimized(false)
	}
}
