// This script REQUIRES that SuperClass.js is included prior to this file.
function Layout(container) {
    this.items=[];
    this.container=(container?container:null);
    this.padding=2;
}

Layout.prototype=new SuperClass();

Layout.prototype.setContainer=function(layer) {
    if(layer.getClass()!=DynLayer) return alert("Cannot add a non DynLayer as layout container");
    this.container=layer;
}

Layout.prototype.add=function(layer) {
    if (this.container==null) return;
    this.container.addChild(layer);
    if (this.container.created)
        this.doLayout(layer)
}

// Basicly this method is a placeholder if the layout doesn't support insertion.
Layout.prototype.insertChild=function(layer,idx) {
    this.addChild(layer);
}

Layout.prototype.setPadding=function(p) {
    this.padding=p;
}

function FlowLayout(container) {
    n=new Layout(container||null);
    n.padding=0
    n.leftPad=3;
    n.rightPad=3;
    n.maxWidth=0;
    n.overflow="wrap";
    n.flowDirection=0; //left -> right    
    n.inherit(FlowLayout); // become a FlowLayout
    return n;
}

FlowLayout.prototype={
    doLayout:function() {
        var x,y,w,h,c,t,start,stop,step;
        x=y=w=h=0;
        if (this.container==null || this.container.children.length==0) return;
        c=this.container.children;
        if (this.flowDirection==0) {
            start=1;stop=c.length+1;step=1;
        } else {
            stop=0;start=c.length;step=-1;
        }
        var pc=this.container.parentComponent;
        var borderPad=pc.border;
        w=(is.ie?0:pc.border)+this.leftPad;
        y=(is.ie?0:pc.border);
        for (var i=start;i!=stop;i+=step) {
            c[i-1].moveTo(w,y);
            w+=c[i-1].getWidth()+this.padding;
            t=c[i-1].getHeight();
            h=(t>h?t:h);
            if (this.maxWidth>0 && w>this.maxWidth) {
                if (this.overflow=="wrap") {
                    y+=h;
                    x=w;
                    h=0;
                    w=this.leftPad;
                }
                else this.container.invokeEvent("overflow");
            }
        }
        this.container.setSize(((x>0?x:w)-this.padding)+this.rightPad+(is.ie?this.leftPad:0)+(borderPad*2),h+(borderPad*2));
        if (this.flowDirection) // if reverse flow, then move the container so that end is at x/y
            this.container.setX(this.container.getX()-this.container.w,this.container.getY());
    },
    setMaxWidth:function(w) {
        this.maxWidth=w;
    }
}

function VertFlowLayout(container) {
    n=new Layout(container||null);
    n.padding=0;
    n.leftPad=0;
    n.rightPad=0;
    n.maxHeight=0;
    n.flowDirection=0; //top -> bottom
    n.inherit(VertFlowLayout); // become a VertFlowLayout
    return n;
}
    
VertFlowLayout.prototype={
    doLayout:function() {
    var x,y,w,h,c,t;
    x=y=w=h=0;
    if (this.container==null || this.container.children.length==0) return;
    c=this.container.children;
    if (this.flowDirection==0) {
        start=1;stop=c.length+1;step=1;
    } else {
        stop=0;start=c.length;step=-1;
    }
    var pc=this.container.parentComponent;
    var borderPad=(pc.childBorder?pc.childBorder:0);
    w=(is.ie?0:borderPad)+this.leftPad;
    y=(is.ie?0:borderPad);
    for (var i=start;i!=stop;i+=step) {
        c[i-1].moveTo(x,h);
        h+=c[i-1].getHeight()+this.padding;
        t=c[i-1].getWidth();
        w=(t>w?t:w);
        if (this.maxHeight>0 && w>this.maxHeight) {
            if (this.overflow=="wrap") {
                x+=w;
                y=h;
                h=0;
                w=this.leftPad;
            }
            else this.container.invokeEvent("overflow");
        }
    }
    this.container.setSize(w+(borderPad*2)+this.rightPad+(is.ie?this.leftPad:0),((y>0?y:h)-this.padding)+(borderPad*2));
    if (this.flowDirection) // if reverse flow, then move the container so that end is at x/y
        this.container.setY(this.container.getX(),this.container.getY()-this.container.h);
    },
    setMaxHeight:function(h) {
        this.maxHeight=h;
    }
}
// Preliminary layout managers who are not yet fully implemented.
function BorderLayout(overlap) {
    n=new Layout();
    n.overlap=(overlap?overlap:false);
    // Order to add if no placement provided.
    n.order=["North","East","South","West","Center"];
    n.pos={};
    n.inherit(BorderLayout); // become a BorderLayout
    return n;
}

BorderLayout.prototype.add=function(layer,loc) {
    if (this.container==null) return;
    if (!loc) {
        var i=0;
        for (i=0;i<this.order.length;i++)
            if (!this.pos[this.order[i]]) break;
        if (i==this.order.length) i=0;
        loc=this.order[i];
    }
    if (this.pos[loc])
        this.pos[loc].removeFromParent();
    this.pos[loc]=layer;
    this.Super.add(layer);
}

function GridLayout(rows,columns) {//Width and height are in cells, not pixels
    n=new Layout();
    n.rows=rows;
    n.cols=columns;
    n.inherit(GridLayout); // become a BorderLayout
    return n;
}

function RelativeLayout() {
    n=new Layout();
    n.pos={};
    n.lParams=[]
    n.baseList=[];
    n.targetList=[];
    n.byType={spring:[],strut:[],fixed:[],relative:[]};
    n.inherit(RelativeLayout); // become a BorderLayout
    return n;
}

RelativeLayout.parameter=function(lBase,lTarget,angle,dist,type) {
    this.base=lBase;
    this.target=lTarget;
    this.angle=angle;
    this.dist=dist;
    this.type=type;
    this.set=false; // This layout has not been executed.
}

RelativeLayout.add=function(lBase,lTarget,angle,dist,type) {
/*
    if (layoutParam!=RelativeLayout.parameter.constructor) {
        alert("RelLayout.add: argument not a relative layout parameter.");
        return;
    }
*/
    var bID,tID;
    if (lBase.constructor!=DynLayer) bID=lBase.getComponent().id;
    else bID=lBase.id;
    if (lTarget.constructor!=DynLayer) tID=lTarget.getComponent().id;
    else tID=lTarget.id;
    var rl=new RelativeLayout.parameter(lBase,lTarget,angle,dist,type);
    rl.index=this.lParams.length;
    this.lParams[rl.index]=rl;
    this.baseList[bID]=lBase; // A Base is like an anchor. They are not moved.. only targets are moved.
    if (!this.depList[tID]) this.depList[tID]=[];
    this.depList[tID][this.depList[tID].length]=rl;
//    this.targetList[tID]=lTarget;
//    this.byType[type][this.byType[type].length]=rl
    this.Super.add(lBase);
    this.Super.add(lTarget);
    
}
// These do NOT need to be part of the function.. they are like private/static methods that are usefull
// to all RelativeLayout objects, and is threadsafe, in the event multi-threading javascript becomes a reality.
RelativeLayout.placeSpring=function(lParam) {
   // var deps[]=getDependencies(lParam.base);
}

RelativeLayout.placeStrut=function(lParam) {
}

RelativeLayout.placeFixed=function(lParam) {
}

RelativeLayout.placeRel=function(lParam) {
}

RelativeLayout.prototype= {
    calculateDependencyLimits:function(target) {
        var l=this.depList[target.id].length;
        var dl=[];
        for(var i=0;i<l;i++) {
            if (this.depList[target.id][i].set) {
                dl[dl.length]=this.depList[target.id][i].angle;
                dl[dl.length]=this.depList[target.id][i].dist;
            }
        }
        var vect=FastMath.addVectors(dl);
    },
    add:function(lBase,lTarget,angle,dist,type) {
    /*
        if (layoutParam!=RelativeLayout.parameter.constructor) {
            alert("RelLayout.add: argument not a relative layout parameter.");
            return;
        }
    */
        var bID,tID;
        if (lBase.constructor!=DynLayer) bID=lBase.getComponent().id;
        else bID=lBase.id;
        if (lTarget.constructor!=DynLayer) tID=lTarget.getComponent().id;
        else tID=lTarget.id;
        var rl=new RelativeLayout.parameter(lBase,lTarget,angle,dist,type);
        rl.index=this.lParams.length;
        this.lParams[rl.index]=rl;
        this.baseList[bID]=lBase; // A Base is like an anchor. They are not moved.. only targets are moved.
        if (!this.depList[tID]) this.depList[tID]=[];
        this.depList[tID][this.depList[tID].length]=rl;
    //    this.targetList[tID]=lTarget;
    //    this.byType[type][this.byType[type].length]=rl
        this.Super.add(lBase);
        this.Super.add(lTarget);
        
    },
    doLayout:function() {
        var cBase,lType; //(Current Base, layout type)
        var i,p;
        for (i=0;i<this.lParams.length;i++) {
            this.lParams[i].set=false; // reset all lParams to not executed
        }
    }
}