// Table
// a spreadsheet-style grid/table component
// 2000.06.01

// Copyright (C) 2000 Dan Steinman
// Redistributable under the terms of the GNU Library General Public License
// Available at http://www.dansteinman.com/dynapi/

function Table(rows,cols) {
	this.rows = rows||0
	this.cols = cols||0
	this.cells = []
	this.component = new DynLayer()

	this.itemListener = new EventListener(this)
	this.itemListener.onselect = function(e) {
		e.getTarget().cellOnSelect(e.getSource())
	}
	this.itemListener.ondeselect = function(e) {
		e.getTarget().cellOnDeselect(e.getSource())
	}
	this.recreate()
}

WidgetInterface.inherit(Table)

Table.prototype.nullCell = {}
Table.prototype.selectedRow = -1
Table.prototype.selectedCol = -1
Table.prototype.selectedItem = Table.prototype.nullItem
Table.prototype.rows = 0
Table.prototype.cols = 0
Table.prototype.cellWidth = 0
Table.prototype.cellHeight = 0

Table.prototype.clear = function() {
	this.selectedIndex = -1
	this.selectedItem = this.nullItem
}
Table.prototype.setCols = function(c) {this.setDataSize(this.rows,c)}
Table.prototype.setRows = function(r) {this.setDataSize(r,this.cols)}
Table.prototype.setDataSize = function(r,c) {
	this.rows = r
	this.cols = c
	this.recreate()
}
Table.prototype.setData = function(d) {
	this.setDataSize(d.length,d[0].length)
	for (var r=0;r<this.rows;r++) {
		for (var c=0;c<this.cols;c++) {
			var oldt = this.cells[r][c].getText()
			var newt = d[r][c]
			if (newt!=oldt) this.cells[r][c].setText(newt)
		}
	}
}
Table.prototype.setCellData = function(cell,data) {
	cell.setText(data)
}
Table.prototype.getCell = function(row,col) {
	if (this.cells[row][col]) return this.cells[row][col]	
	else return null
}
Table.prototype.getCells = function() {
	return this.cells
}
Table.prototype.getRowCells = function(r) {
	if (this.cells[r]) return this.cells[r]
	else return []
}
Table.prototype.getColCells = function(c) {
	if (this.cells[0][c]) {
		var temp = []
		for (var r=0;r<this.rows;r++) {
			temp[r] = this.cells[r][c]
		}
		return temp
	}
	else return []	
}
Table.prototype.getRowOf = function(cell) {
	for (var r=0;r<this.rows;r++) {
		for (var c=0;c<this.cols;c++) {
			if (cell == this.cells[r][c]) return r
		}
	}
}
Table.prototype.getColOf = function(cell) {
	for (var r=0;r<this.rows;r++) {
		for (var c=0;c<this.cols;c++) {
			if (cell == this.cells[r][c]) return c
		}
	}
}
Table.prototype.select = function(r,c) {
	this.selectCell(this.getCell(r,c))
}
Table.prototype.selectCell = function(cell) {
	cell.setSelected(true)
}
Table.prototype.selectRow = function(r) {
	this.multipleSelect = true
	this.selectedRow = r
	var cells = this.getRowCells(r)
	for (var i in cells) {
		cells[i].setSelected(true)
	}
	this.multipleSelect = false
}
Table.prototype.selectCol = function(c) {
	this.multipleSelect = true
	this.selectedCol = c
	var cells = this.getColCells(c)
	for (var i in cells) {
		cells[i].setSelected(true)
	}
	this.multipleSelect = false
}
Table.prototype.cellOnSelect = function(cell) {	
	for (var r=0;r<this.rows;r++) {
		for (var c=0;c<this.cols;c++) {
			if (cell == this.cells[r][c]) {
				this.selectedRow = r
				this.selectedCol = c
				this.selectedCell = cell
				this.selectedData = cell.getData()
			}
			else if (!this.multipleSelect) this.cells[r][c].setSelected(false)
		}
	}
	this.invokeEvent("select")
}
Table.prototype.cellOnDeselect = function(cell) {
	if (this.selectedCell == cell) {
		this.selectedRow = -1
		this.selectedCol = -1
		this.selectedCell = null
		this.selectedData = ''
	}
}
Table.prototype.recreate = function() {
	for (var r=0;r<this.rows;r++) {
		if (!this.cells[r]) this.cells[r] = []
		for (var c=0;c<this.cols;c++) {
			if (!this.cells[r][c]) {
				this.cells[r][c] = new TableCell()
				this.component.addChild(this.cells[r][c])
				this.cells[r][c].addEventListener(this.itemListener)
			}
			this.cells[r][c].moveTo(this.cellWidth*c,this.cellHeight*r)
			this.cells[r][c].setAlignment(this.cellAlignment)
			this.cells[r][c].setSize(this.cellWidth,this.cellHeight)
		}
	}
	this.component.setSize(this.cellWidth*this.cols,this.cellHeight*this.rows)	
}
Table.prototype.setCellAlignment = function(a) {
	this.cellAlignment = a
	this.recreate()
}
Table.prototype.setCellWidth = function(w) {this.setCellSize(w,null)}
Table.prototype.setCellHeight = function(h) {this.setCellSize(null,h)}
Table.prototype.setCellSize = function(w,h) {
	if (w!=null) this.cellWidth = w	
	if (h!=null) this.cellHeight = h
	this.recreate()
}

TableCell = ListItem
// not good, adds those methods to ListItem also
TableCell.prototype.getData = function() {return this.getText()}
TableCell.prototype.setData = function(d) {this.setText(d)}
